"use strict";
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
var __spreadArray = (this && this.__spreadArray) || function (to, from) {
    for (var i = 0, il = from.length, j = to.length; i < il; i++, j++)
        to[j] = from[i];
    return to;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Head = exports.Default = exports.Case = exports.Switch = exports.If = exports.HtmlPage = exports.Fragment = exports.create = exports.render = exports.Element = exports.createElement = void 0;
/**
 * Implementation.
 */
function createElement(nameOrGenerator, props) {
    var children = [];
    for (var _i = 2; _i < arguments.length; _i++) {
        children[_i - 2] = arguments[_i];
    }
    if (typeof nameOrGenerator == "function") {
        return nameOrGenerator(props !== null && props !== void 0 ? props : {}, children);
    }
    else if (nameOrGenerator == exports.Fragment) {
        return new Element(exports.Fragment, {}, children);
    }
    else {
        return new Element(nameOrGenerator, props !== null && props !== void 0 ? props : {}, children);
    }
}
exports.createElement = createElement;
/**
 * Readonly data container for JSX elements.
 */
var Element = /** @class */ (function () {
    /**
     * Creates a new element data container.
     * @param name The name of the element. (Like `div`, `a`, etc.)
     * @param props The properties/attributes assigned to the element.
     * @param children Children of the element.
     */
    function Element(name, props, children) {
        this.name = name;
        this.props = props;
        this.children = flatten(children);
    }
    return Element;
}());
exports.Element = Element;
function flatten(children) {
    var result = [];
    children.forEach(function (c) {
        if (c instanceof Array)
            result.push.apply(result, flatten(c));
        else if (c instanceof Element && c.name == exports.Fragment)
            result.push.apply(result, flatten(c.children));
        else
            result.push(c);
    });
    return result;
}
/**
 * Renders the JSX elements to a string. This function provides the core functionallity.
 * @param top The first (top) element that should be rendered.
 * @param options Options to use for rendering.
 * @param indentation The starting indentation level. Only used when options.indent is set to `true`.
 */
function render(top, options, indentation) {
    if (options === void 0) { options = {}; }
    if (indentation === void 0) { indentation = 0; }
    options = __assign({ indent: false, indentString: "", indentSize: 4, useSelfCloseTags: true }, options);
    return renderInternal(top, options, indentation);
}
exports.render = render;
function renderInternal(node, options, indentation) {
    if (typeof node == "string") {
        return createIndent(options, indentation) + node;
    }
    else if (node instanceof Array) {
        return node.map(function (n) { return renderInternal(n, options, indentation); }).join(options.indent ? "\n" : "");
    }
    else if (node.name == exports.Fragment) {
        return node.children.map(function (n) { return renderInternal(n, options, indentation); }).join(options.indent ? "\n" : "");
    }
    else if (node.name == HTML) {
        var indentString = createIndent(options, indentation);
        var children = __spreadArray([], node.children);
        var index = children.findIndex(function (n) { return n instanceof Element && n.name == HEAD; });
        var head = void 0;
        if (index == -1)
            head = [];
        else {
            head = children[index].children;
            children.splice(index, 1);
        }
        var result = indentString + "<!DOCTYPE html>" + (options.indent ? "\n" : "");
        result += indentString + "<html>" + (options.indent ? "\n" : "");
        result += indentString + "<head>" + (options.indent ? "\n" : "");
        result += renderInternal(head, options, indentation + 1) + (options.indent && head.length > 0 ? "\n" : "");
        result += indentString + "</head>" + (options.indent ? "\n" : "");
        result += indentString + "<body>" + (options.indent ? "\n" : "");
        result += children.map(function (n) { return renderInternal(n, options, indentation + 1); }).join(options.indent ? "\n" : "");
        if (children.length > 0 && options.indent)
            result += "\n";
        result += indentString + "</body>" + (options.indent ? "\n" : "");
        result += indentString + "</html>";
        return result;
    }
    else if (node.name == HEAD) {
        return "";
    }
    else if (node.name == CASE || node.name == DEFAULT) {
        return renderInternal(node.children, options, indentation);
    }
    else {
        var selfClose = node.children.length == 0 && (options.useSelfCloseTags == true || (options.useSelfCloseTags instanceof Array && options.useSelfCloseTags.includes(node.name)));
        if (node.props.className && !node.props.class)
            node.props.class = node.props.className;
        var props = __assign({}, node.props);
        if (props.className && !props.class)
            props.class = props.className;
        delete props.className;
        var attrs = createAttrs(node.props);
        var indentString = createIndent(options, indentation);
        var result = indentString + "<" + node.name + attrs + (selfClose ? " /" : "") + ">";
        if (!selfClose) {
            if (node.children.length > 0 && options.indent)
                result += "\n";
            result += node.children.map(function (n) { return renderInternal(n, options, indentation + 1); }).join(options.indent ? "\n" : "");
            if (node.children.length > 0 && options.indent)
                result += "\n" + indentString;
            result += "</" + node.name + ">";
        }
        return result;
    }
}
function createIndent(options, indentation) {
    if (!options.indent)
        return "";
    if (options.indentString != "")
        return options.indentString.repeat(indentation);
    if (options.indentSize != 0)
        return " ".repeat(indentation * options.indentSize);
    return "";
}
function createAttrs(props) {
    var attrKeys = Object.keys(props);
    var attrs = [];
    attrKeys.forEach(function (k) {
        attrs.push(" " + k + (props[k] === true ? "" : "=\"" + props[k] + "\""));
    });
    return attrs.join("");
}
function create(options) {
    if (options === void 0) { options = {}; }
    if (options.useSelfCloseTags == null)
        options.useSelfCloseTags = [
            "area",
            "base",
            "br",
            "col",
            "embed",
            "hr",
            "img",
            "input",
            "link",
            "input",
            "link",
            "meta",
            "param",
            "source",
            "track",
            "wbr"
        ];
    var renderer = Object.assign(function (top) {
        if (top.name != HTML)
            top = new Element(HTML, {}, [top]);
        return render(top, options);
    }, {
        createHandler: function (component, status) {
            return function (req, res) {
                var element = createElement(component, { req: req, res: res });
                if (element.name != HTML)
                    element = new Element(HTML, {}, [element]);
                var actualStatus;
                if (status != null) {
                    actualStatus = status;
                }
                else if (typeof element.props.status == "number") {
                    actualStatus = element.props.status;
                }
                else {
                    actualStatus = 200;
                }
                res.status(actualStatus).send(renderer(element));
            };
        }
    });
    return renderer;
}
exports.create = create;
/**
 * Symbol indicating that the element is a fragment.
 */
exports.Fragment = Symbol("Fragment");
/**
 * Element for an HTML page (including DOCTYPE, head and body).
 */
var HtmlPage = function (props, children) {
    return new Element(HTML, props, children);
};
exports.HtmlPage = HtmlPage;
var HTML = Symbol("HTML");
/**
 * Helper component for conditional elements.
 */
var If = function (_a, children) {
    var cond = _a.cond;
    if (cond)
        return new Element(exports.Fragment, {}, children);
    else
        return new Element(exports.Fragment, {}, []);
};
exports.If = If;
/**
 * Helper component for multiple case szenarios.
 */
var Switch = function (_a, children) {
    var expr = _a.expr;
    var found = children.find(function (c) { return c instanceof Element && c.name == CASE && c.props.c == expr; });
    if (found)
        return new Element(exports.Fragment, {}, found.children);
    var def = children.find(function (c) { return c instanceof Element && c.name == DEFAULT; });
    if (def)
        return new Element(exports.Fragment, {}, def.children);
    return new Element(exports.Fragment, {}, []);
};
exports.Switch = Switch;
var Case = function (_a, children) {
    var c = _a.c;
    return new Element(CASE, { c: c }, children);
};
exports.Case = Case;
var Default = function (_, children) {
    return new Element(DEFAULT, {}, children);
};
exports.Default = Default;
var CASE = Symbol("Case");
var DEFAULT = Symbol("Default");
var Head = function (props, children) {
    return new Element(HEAD, props, children);
};
exports.Head = Head;
var HEAD = Symbol("Head");
exports.default = {
    createElement: createElement,
    Element: Element,
    render: render,
    Fragment: exports.Fragment,
    HtmlPage: exports.HtmlPage,
    create: create,
};
//# sourceMappingURL=index.js.map